<?php

namespace ext\crudactions;

/**
 * @property string $modelClass Name of model class
 * @property string $modelPostName Model post name
 * @property string $modelIdentifier Model Identifier
 * @property string $viewFile View file
 */
abstract class Base extends \CAction {

    /**
     * @var string Name of model class
     */
    protected $_modelClass;

    /**
     * @var string Model post name
     */
    protected $_modelPostName;

    /**
     * @var string Model Identifier
     */
    protected $_modelIdentifier;

    /**
     * @var string View file
     */
    protected $_viewFile;

    /**
     * @var bool Enable ajax validation
     */
    public $enableAjaxValidation = true;

    /**
     * @var array Attributes/columns that should be displayed in action's view.
     */
    public $displayColumns;

    /**
     * @var string Name of active form class. Must be or extend from \CActiveForm. (default: 'CActiveForm')
     */
    public $activeFormClass = 'CActiveForm';

    /**
     * Raises onBeforeRender event
     *
     * @param \CEvent $event
     */
    public function onBeforeRender(\CEvent $event) {
        $this->raiseEvent('onBeforeRender', $event);
    }

    /**
     * Raises onAfterRender event
     *
     * @param \CEvent $event
     */
    public function onAfterRender(\CEvent $event) {
        $this->raiseEvent('onAfterRender', $event);
    }

    /**
     * Raises onBeforeSave event. The event provides the model instance used.
     *
     * @param Event $event
     */
    public function onBeforeSave(Event $event) {
        $this->raiseEvent('onBeforeSave', $event);
    }

    /**
     * Raises onAfterSave event. The event provides the model instance used.
     *
     * @param Event $event
     */
    public function onAfterSave(Event $event) {
        $this->raiseEvent('onAfterSave', $event);
    }

    /**
     * Getter for _modelClass
     *
     * @return string
     * @throws \CException
     */
    public function getModelClass() {
        if (empty($this->_modelClass)) {
            $this->_modelClass = ucfirst($this->getController()->getId());
            if (!class_exists($this->_modelClass))
                throw new \CException('The model class must be defined for ' . $this->getController()->getId() . '::' . $this->getId() . ' (' . get_called_class($this) . '::modelClass)');
        }
        return $this->_modelClass;
    }

    /**
     * Setter for _modelClass
     *
     * @param string $str Model class name
     */
    public function setModelClass($str) {
        $this->_modelClass = $str;
    }

    /**
     * Getter for _modelPostName
     *
     * @return string
     */
    public function getModelPostName() {
        if (empty($this->_modelPostName))
            $this->_modelPostName = $this->modelClass;
        return $this->_modelPostName;
    }

    /**
     * Setter for _modelPostName
     *
     * @param string $str
     */
    public function setModelPostName($str) {
        $this->_modelPostName = $str;
    }

    /**
     * Getter for _modelIdentifier
     *
     * @return string
     */
    public function getModelIdentifier() {
        if (empty($this->_modelIdentifier))
            $this->_modelIdentifier = strtolower(get_class($this->getController())) . '-' . strtolower($this->modelClass);
        return $this->_modelIdentifier;
    }

    /**
     * Setter for _modelIdentifier
     *
     * @param string $str
     */
    public function setModelIdentifier($str) {
        $this->_modelIdentifier = $str;
    }

    /**
     * Getter for _viewFile
     *
     * @return string
     */
    public function getViewFile() {
        if (empty($this->_viewFile)) {
            $id = strtolower(str_replace(__NAMESPACE__.'\\', '', get_class($this)));
            $this->_viewFile = 'ext.crudactions.views.' . $id;
        }
        return $this->_viewFile;
    }

    /**
     * Setter for _viewFile
     *
     * @param string $str
     */
    public function setViewFile($str) {
        $this->_viewFile = $str;
    }

    /**
     * Internal render method, raises before/after render events.
     * Passes render call on up to controller.
     *
     * @param string $viewFile
     * @param array $params
     */
    protected function render($viewFile, array $params = array()) {
        /**
         * @todo May be able to convince Yii core to implement render events
         */
        $this->onBeforeRender(new \CEvent($this));
        $this->getController()->render($viewFile, $params);
        $this->onAfterRender(new \CEvent($this));
    }

    /**
     * Internal method for performing ajax validation.
     *
     * @param \CModel $model
     */
    protected function performAjaxValidation(\CModel $model) {
        if (isset($_POST['ajax']) && $_POST['ajax'] === $this->modelIdentifier . '-form') {
            $formClass = $this->activeFormClass;
            echo $formClass::validate($model);
            \Yii::app()->end();
        }
    }

    /**
     * Internal method for handling validating/saving.
     *
     * @param \CModel $model
     */
    protected function save(\CModel $model) {
        if (empty($this->modelPostName))
            $this->modelPostName = $this->modelClass;

        if ($this->enableAjaxValidation)
            $this->performAjaxValidation($model);

        if (isset($_POST[$this->modelPostName])) {
            $model->attributes = $_POST[$this->modelPostName];

            /**
             * Raise new event for before save
             */
            $this->onBeforeSave(new Event($model, $this));
            $successful = $model->save();
            /**
             * Attempt to save model, pass result into event, raise after save event
             */
            $this->onAfterSave(new Event($model, $this, $successful));

            if ($successful)
                $this->getController()->redirect(array('view', 'id' => $model->getPrimaryKey()));
        }
    }

    /**
     * Generates html input field, using \CHtml class.
     *
     * @param \CModel $model Model instance
     * @param string $column Column name to be rendered as input
     * @return string
     */
    public function generateInputField(\CModel $model, $column) {
        if ($column->type === 'boolean')
            return \CHtml::activeCheckBox($model, $column->name);
        else if (stripos($column->realType, 'text') !== false)
            return \CHtml::activeTextArea($model, $column->name, array('rows' => 6, 'cols' => 50));
        else {
            if (preg_match('/^(password|pass|passwd|passcode)$/i', $column->name))
                $inputField = 'activePasswordField';
            else
                $inputField = 'activeTextField';

            if ($column->type !== 'string' || $column->size === null)
                return \CHtml::$inputField($model, $column->name);
            else {
                if (($size = $maxLength = $column->size) > 60)
                    $size = 60;
                return \CHtml::$inputField($model, $column->name, array('size' => $size, 'maxlength' => $maxLength));
            }
        }
    }

    /**
     * Generates active html input field, using \CActiveForm class (or derivative).
     *
     * @param \CActiveForm $form Active form instance
     * @param \CModel $model Model instance
     * @param string $column Column name to be rendered as input
     * @return string
     */
    public function generateActiveField(\CActiveForm $form, \CModel $model, $column) {
        if ($column->type === 'boolean')
            return $form->checkBox($model, $column->name);
        else if (stripos($column->realType, 'text') !== false)
            return $form->textArea($model, $column->name, array('rows' => 6, 'cols' => 50));
        else {
            if (preg_match('/^(password|pass|passwd|passcode)$/i', $column->name))
                $inputField = 'passwordField';
            else
                $inputField = 'textField';

            if ($column->type !== 'string' || $column->size === null)
                return $form->$inputField($model, $column->name);
            else {
                if (($size = $maxLength = $column->size) > 60)
                    $size = 60;
                return $form->$inputField($model, $column->name, array('size' => $size, 'maxlength' => $maxLength));
            }
        }
    }

    /**
     * Internal method for loading a requested model instance
     *
     * @param mixed $id
     * @return \CModel
     * @throws \CHttpException
     */
    protected function loadModel($id) {
        $_m = $this->modelClass;
        $model = $_m::model()->findByPk($id);
        if ($model === null)
            throw new \CHttpException(404, 'The requested page does not exist.');
        return $model;
    }

}